#include "StdAfx.h"
#include "ChartWrapper.h"

#include "Aspect.h"
#include "BarSeries.h"
#include "LineSeries.h"
#include "Brush.h"
#include "Canvas.h"
#include "Marks.h"
#include "Pen.h"
#include "PointSeries.h"
#include "Pointer.h"
#include "Scroll.h"
#include "Series.h"
#include "TeeFunction.h"
#include "Zoom.h"
#include "tchart.h"
#include "axes.h"
#include "axis.h"
#include "axislabels.h"
#include "axistitle.h"
#include "titles.h"
#include "legend.h"
#include "export.h"
#include "toollist.h"
#include "tools.h"
#include "rotatetool.h"
#include "printer.h"
#include "bmpexport.h"
#include "pngexport.h"
#include "jpegexport.h"
#include "horizbarseries.h"
#include "panel.h"
#include "gradient.h"
#include "walls.h"
#include "wall.h"
#include "teefont.h"
#include "bezierseries.h"
#include "strings.h"
#include "areaseries.h"
#include "environment.h"
#include "chartaxispen.h"
#include "toollist.h"
#include "tools.h"
#include "markstiptool.h"
#include "ValueList.h"
#include <math.h>
#include <set>
#include "smithseries.h"
#include "polarseries.h"

const int CChartWrapper::LINE_TYPE = 0;
const int CChartWrapper::BAR_TYPE = 1;
const int CChartWrapper::H_BAR_TYPE = 2;
const int CChartWrapper::POINT_TYPE = 4;
const int CChartWrapper::BEZIER_TYPE = 16;
const int CChartWrapper::AREA_TYPE = 3;
const int CChartWrapper::SMITH_TYPE = 34;
const int CChartWrapper::POLAR_TYPE = 12;


double GetNearestLogPoint(const double val)
{
	
	double temp = val;
	int x = 1;

	double result = 0;

	if(val == 0)
	{
		result = 1;
		return result;
	}

	if(val < 1)
	{
		while(temp*10 <= 1)
		{
			x++;
			temp = temp*10;
		}
		result = pow(10.0,x);
		result = 10/result;
	}
	else
	{
		int iVal = static_cast<int>(val);
		if((iVal % 10) != 0)
		{
			while((iVal / 10) >= 1)
			{
				x++;
				iVal = iVal/10;
			}

			return result = pow(10.0,x);
		}

		result = val;
		
	}
	return result;
	
	
}

CChartWrapper::CChartWrapper(CTChart *chart) : pchart_(chart),
plot_count_(0),
isAreaActive_(false),
bAutoscale_(false),
isDynamicallyCreated_(false)
{
}

CChartWrapper::~CChartWrapper(void)
{
	if ( font_.get() ) {
		font_->DeleteObject();
	}

	if (isDynamicallyCreated_)
	{
		if (pchart_) {
			delete pchart_;
			pchart_ = 0;
		}
	}
	
}

CTChart* CChartWrapper::GetTChart()
{
	return pchart_;
}

bool CChartWrapper::CreateTChart(const CRect &rect,
								CWnd *parent,
								const UINT res_id)
{
	pchart_ = new CTChart();
	try
	{
		BOOL bStatus{ FALSE };

		while (true) {

			bStatus = pchart_->Create(_T(""), _T(""), WS_CHILD | WS_VISIBLE, rect, parent, res_id);

			if (bStatus) {
				break;
			}

			::Sleep(50);
		}

		if (bStatus) {

			while (true) {

				if (pchart_->GetSafeHwnd()) {
					break;
				}

				::Sleep(100);
			}
		}

		res_id_ = res_id;
	}
	catch(...)
	{
	}

	pchart_->RemoveAllSeries();

	isDynamicallyCreated_ = true;
	
	return true;
}

bool CChartWrapper::SetTChartProperty()
{
	set_axes_.clear();
	set_axes_.insert(EN_AXIS_HORIZONTAL_BOTTOM); // 
	set_axes_.insert(EN_AXIS_VERTICAL_LEFT);

	pchart_->GetAspect().SetView3D(false);
	pchart_->GetHeader().SetVisible(false);
	pchart_->GetLegend().SetVisible(FALSE);
	pchart_->SetClipPoints(true);
	pchart_->GetAxis().SetDrawAxesBeforeSeries(true);
	pchart_->SetTimerInterval(0);
	pchart_->GetAspect().SetHorizOffset(1) ;
	pchart_->GetAspect().SetVertOffset(1);
	pchart_->GetScroll().SetEnable(false);
	pchart_->GetLegend().SetVisible(FALSE);
	pchart_->GetAxis().GetBottom().SetVisible(true);
	pchart_->GetAxis().GetLeft().SetVisible(true);
	pchart_->GetAxis().GetLeft().SetMinimum(0);
	pchart_->GetAxis().GetLeft().SetMaximum(10);
	pchart_->GetAxis().GetBottom().SetMinimum(0);
	pchart_->GetAxis().GetBottom().SetMaximum(10);

	pchart_->GetAxis().GetRight().SetVisible(false);
	pchart_->GetAxis().GetTop().SetVisible(false);
	pchart_->GetAxis().GetBottom().SetAutomatic(false);
	pchart_->GetAxis().GetLeft().SetAutomatic(false);

	SetAxisFont(EN_AXIS_HORIZONTAL_BOTTOM, "Arial", 8);
	SetAxisFont(EN_AXIS_VERTICAL_LEFT, "Arial", 8);

	pchart_->GetEnvironment().SetMouseWheelScroll(FALSE);
	pchart_->StopMouse();
	pchart_->GetZoom().SetEnable(FALSE);
	pchart_->GetZoom().SetMouseButton(FALSE);
	prev_plotid_ = -1;

	xExpo_ = EN_LABLE_NONE;
	y1Expo_ = EN_LABLE_NONE;
	y2Expo_ = EN_LABLE_NONE;

	bInitX_ = true;
	bInitY2_ = true;
	bInitY1_ = true;

	return true;
}

void CChartWrapper::GetTChartWindowRect(CRect& rect)
{
	pchart_->GetWindowRect(rect);
}

void CChartWrapper::TChartInvalidate()
{
	pchart_->Invalidate();
	pchart_->UpdateWindow();
	pchart_->RedrawWindow();
	pchart_->Repaint();
	pchart_->RefreshData();
	pchart_->EnableToolTips(true);
	pchart_->EnableTrackingToolTips(true);
}

bool CChartWrapper::IsAxisLog(const enAxis axis)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			if (pchart_->GetAxis().GetBottom().GetLogarithmic() == TRUE)
				return true;
			else
				return false;

		case EN_AXIS_HORIZONTAL_TOP:
			set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
			if (pchart_->GetAxis().GetTop().GetLogarithmic() == TRUE)
				return true;
			else
				return false;
			
		case EN_AXIS_VERTICAL_LEFT:
			if (pchart_->GetAxis().GetLeft().GetLogarithmic() == TRUE)
				return true;
			else
				return false;
		case EN_AXIS_VERTICAL_RIGHT:
			set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
			if (pchart_->GetAxis().GetRight().GetLogarithmic() == TRUE)
				return true;
			else
				return false;
		default:
			return false;
		
	}
}

void CChartWrapper::SetAxisLog(const enAxis axis, const bool isLog)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			pchart_->GetAxis().GetBottom().SetLogarithmic(isLog);
			pchart_->GetAxis().GetBottom().SetLogarithmicBase(10);
			break;
		case EN_AXIS_HORIZONTAL_TOP:
			pchart_->GetAxis().GetTop().SetLogarithmic(isLog);
			pchart_->GetAxis().GetTop().SetLogarithmicBase(10);
			set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
			break;
		case EN_AXIS_VERTICAL_LEFT:
			pchart_->GetAxis().GetLeft().SetLogarithmic(isLog);
			pchart_->GetAxis().GetLeft().SetLogarithmicBase(10);
			set_axes_.insert(EN_AXIS_VERTICAL_LEFT);
			break;
		case EN_AXIS_VERTICAL_RIGHT:
			pchart_->GetAxis().GetRight().SetLogarithmic(isLog);
			pchart_->GetAxis().GetRight().SetLogarithmicBase(10);
			break;
		default:
			return;		
	}
}

void CChartWrapper::SetAxisTitle(const enAxis axis,const CString &title)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			pchart_->GetAxis().GetBottom().GetTitle().GetFont().SetName("Arial");
			pchart_->GetAxis().GetBottom().GetTitle().GetFont().SetSize(9);
			pchart_->GetAxis().GetBottom().GetTitle().SetCaption(title);
		    pchart_->GetAxis().GetBottom().GetMinorTicks().SetVisible(false);
		break;

		case EN_AXIS_HORIZONTAL_TOP:
			pchart_->GetAxis().GetTop().GetTitle().GetFont().SetName("Arial");
			pchart_->GetAxis().GetTop().GetTitle().GetFont().SetSize(9);
			pchart_->GetAxis().GetTop().GetTitle().SetCaption(title);
			set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
		break;

		case EN_AXIS_VERTICAL_LEFT:
			pchart_->GetAxis().GetLeft().GetTitle().GetFont().SetName("Arial");
			pchart_->GetAxis().GetLeft().GetTitle().GetFont().SetSize(9);
			pchart_->GetAxis().GetLeft().GetTitle().SetCaption(title);
			pchart_->GetAxis().GetLeft().GetTicks().SetVisible(true);
			
		break;

		case EN_AXIS_VERTICAL_RIGHT:
			pchart_->GetAxis().GetRight().GetTitle().GetFont().SetName("Arial");
			pchart_->GetAxis().GetRight().GetTitle().GetFont().SetSize(9);
			pchart_->GetAxis().GetRight().GetTitle().SetCaption(title);
			pchart_->GetAxis().GetRight().GetTitle().SetAngle(90);
			pchart_->GetCanvas().SetTextAlign(1);
			set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
		break;	
	}
}

int CChartWrapper::AddPlot(const CString &strParameter,
						   const double width, 
						   const long axis)
{
	int plot_id = pchart_->AddSeries(LINE_TYPE);

    LPCTSTR  szTemp  = strParameter;
    
	pchart_->Series(plot_id).GetAsLine().GetPointer().SetStyle(psSmallDot);
	pchart_->Series(plot_id).GetAsLine().GetPointer().SetVisible(TRUE);
	pchart_->Series(plot_id).GetAsLine().GetLinePen().SetWidth(static_cast<long>(width));
	pchart_->Series(plot_id).SetTitle(szTemp);

	set_plot_.insert(plot_id);
    mapOfLegends_.insert(std::make_pair(strParameter,plot_id));

	pchart_->Series(plot_id).SetVerticalAxis(axis);

	return plot_id;
}

int CChartWrapper::AddPointPlot(const CString &strParameter,
						   const double /*width*/,
						   const long axis)
{
	int plot_id = pchart_->AddSeries(POINT_TYPE);

    LPCTSTR  szTemp  = strParameter;
    
	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetStyle(psCircle);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetVisible(FALSE);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetVisible(TRUE);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetInflateMargins(false);

	pchart_->Series(plot_id).SetTitle(szTemp);

	set_point_plot_.insert(plot_id);
    mapOfLegends_.insert(std::make_pair(strParameter,plot_id));

	pchart_->Series(plot_id).SetVerticalAxis(axis);

	return plot_id;
}

int CChartWrapper::AddAreaPlot(const CString &/*strParameter*/,
						   const double /*width*/, 
						   const long axis)
{
	int plot_id = pchart_->AddSeries(AREA_TYPE);

	pchart_->Series(plot_id).SetActive(FALSE);
	pchart_->Series(plot_id).GetAsArea().GetLinePen().SetVisible(false);

	set_plot_.insert(plot_id);

	pchart_->Series(plot_id).SetVerticalAxis(axis);

	return plot_id;
}

int CChartWrapper::AddLinePointPlotEx(const CString &strParameter,
						        const double width,
								const long axis)
{
	int plot_id = AddAreaPlot(strParameter,width,axis);
	plot_id = AddPlot(strParameter,width,axis);
	plot_id = AddPointPlot(strParameter,width,axis);

	/*if(axis == 0)
	{
		bInitX_ = true;
		bInitY2_ = true;
		bInitY1_ = true;
	}*/

	return plot_id;
}

void CChartWrapper::RemoveAllPlots()
{
	long sz = pchart_->GetSeriesCount();
	for (int i = 0; i <= sz; ++i)
	{
		//pchart_->Series(i).FreeDatasource(FALSE);
		pchart_->Series(i).Clear();
	}
	pchart_->RemoveAllSeries();
	mapIndex_.clear();
	set_plot_.clear();
	set_point_plot_.clear();
	set_line_point_plot_.clear();
	mapOfLegends_.clear();

	bInitX_ = true;
	bInitY2_ = true;
	bInitY1_ = true;

	mapSmithChartWnd_.clear();
	mapPolarChartWnd_.clear();
}

void CChartWrapper::PlotPointXvsY1D(const int plot_id,
	const Array1D_Double &vec_xval,
	const Array1D_Double &vec_yval)
{
	if (plot_id < 0)
		return;


	if (vec_xval.size() != vec_yval.size())
		return;

	std::deque<double>::const_iterator x_iter = vec_xval.array_.begin();
	std::deque<double>::const_iterator y_iter = vec_yval.array_.begin();

	COLORREF color = RGB(0, 0, 0);
	std::map<int, COLORREF>::iterator map_iter = map_plot_point_color_.find(plot_id);
	if (map_iter != map_plot_point_color_.end())
		color = map_iter->second;

	for (; x_iter != vec_xval.array_.end() && y_iter != vec_yval.array_.end();
		++x_iter, ++y_iter)
	{
		PlotXY(plot_id, *x_iter, *y_iter, color);
	}
}

void CChartWrapper::PlotLineXvsY1D(const int plot_id,
	const Array1D_Double &vec_xval,
	const Array1D_Double &vec_yval)
{

	if (plot_id < 0)
		return;


	if (vec_xval.size() != vec_yval.size())
		return;


	std::deque<double>::const_iterator x_iter = vec_xval.array_.begin();
	std::deque<double>::const_iterator y_iter = vec_yval.array_.begin();


	COLORREF color = RGB(0, 0, 0);
	std::map<int, COLORREF>::iterator map_iter = map_plot_line_color_.find(plot_id);
	if (map_iter != map_plot_line_color_.end())
		color = map_iter->second;
	if (color == RGB(255, 255, 255))
		return;

	for (; x_iter != vec_xval.array_.end() && y_iter != vec_yval.array_.end();
		++x_iter, ++y_iter)
	{
		PlotXY(plot_id, *x_iter, *y_iter, color);
	}

}

void CChartWrapper::PlotXY(const int plot_id, 
						   const double x,
						   const double y,
						   const COLORREF color)
{
	if (plot_id < 0)
		return;
	
	CString xstr(_T(""));

	enAxis yAxis = GetVerticalAxis(plot_id);

	if(prev_plotid_ != plot_id)
	{	
		switch (xExpo_)
		{
			case EN_LABLE_NEGETIVE_EXPONENTIAL:
				SetAxisExpoLabels(CChartWrapper::EN_AXIS_HORIZONTAL_BOTTOM, true, true);
				break;

			case EN_LABLE_POSITIVE_EXPONENTIAL:
				SetAxisExpoLabels(CChartWrapper::EN_AXIS_HORIZONTAL_BOTTOM, true);
				break;

			case EN_LABLE_NORMAL:
				SetAxisExpoLabels(CChartWrapper::EN_AXIS_HORIZONTAL_BOTTOM);
				break;

			default:
			{
				if (x <= 9e-5)
					SetAxisExpoLabels(CChartWrapper::EN_AXIS_HORIZONTAL_BOTTOM, true, true);
				else if (x >= 1e+5)
					SetAxisExpoLabels(CChartWrapper::EN_AXIS_HORIZONTAL_BOTTOM, true);
				else
					SetAxisExpoLabels(CChartWrapper::EN_AXIS_HORIZONTAL_BOTTOM);
			}
			break;
		}
	
		enLableStyle yExpo = EN_LABLE_NONE;
		switch (yAxis)
		{
			case EN_AXIS_VERTICAL_LEFT:
				yExpo = y1Expo_;
				break;

			case EN_AXIS_VERTICAL_RIGHT:
				yExpo = y2Expo_;
				break;
		}

		switch (yExpo)
		{
			case EN_LABLE_NEGETIVE_EXPONENTIAL:
				SetAxisExpoLabels(yAxis, true, true);
				break;

			case EN_LABLE_POSITIVE_EXPONENTIAL:
				SetAxisExpoLabels(yAxis, true);
				break;

			case EN_LABLE_NORMAL:
				SetAxisExpoLabels(yAxis);
				break;

			default:
			{
				if (y <= 9e-5) /*9.9999*/
					SetAxisExpoLabels(yAxis, true, true);
				else if (y >= 1e+5)
					SetAxisExpoLabels(yAxis, true);
				else
					SetAxisExpoLabels(yAxis);
			}
			break;
		}

		prev_plotid_ = plot_id;
	}

	SetScaleX(plot_id, x);

	long index = pchart_->Series(plot_id).AddXY(x, y, xstr, color);
	
	mapIndex_.insert(std::make_pair(x,std::make_pair(plot_id,index)));    	
}

void CChartWrapper::SetScaleX(const int /*plot_id*/, const double x)
{
	if(IsAxisLog(CChartWrapper::EN_AXIS_HORIZONTAL_BOTTOM) && bAutoscale_)
	{
		double high = GetNearestLogPoint(x);
		double low = high / 10;
	
		if(bInitX_)
		{
			pchart_->GetAxis().GetBottom().SetMaximum(high);
			pchart_->GetAxis().GetBottom().SetMinimum(low);
		}
		else
		{
			double prev_max = pchart_->GetAxis().GetBottom().GetMaximum();
			if(high >= prev_max)
				pchart_->GetAxis().GetBottom().SetMaximum(high);

			double prev_min = pchart_->GetAxis().GetBottom().GetMinimum();
			if(low <= prev_min)
				pchart_->GetAxis().GetBottom().SetMinimum(low);
		}
		bInitX_ = false;
	}	
}

void CChartWrapper::SetScaleY(const int plot_id, const double ymin, const double ymax)
{
	if (plot_id < 0)
		return;

	enAxis yAxis = GetVerticalAxis(plot_id);

	switch (yAxis)
	{
		case EN_AXIS_VERTICAL_LEFT:
		{
			if (IsAxisLog(CChartWrapper::EN_AXIS_VERTICAL_LEFT) && bAutoscale_)
			{
				double high = GetNearestLogPoint(ymax);
				double low = ymin;
				if (bInitY1_)
				{
					pchart_->GetAxis().GetLeft().SetMaximum(high);
					pchart_->GetAxis().GetLeft().SetMinimum(low);
				}
				else
				{
					double prev_max = pchart_->GetAxis().GetLeft().GetMaximum();
					if (high >= prev_max)
						pchart_->GetAxis().GetLeft().SetMaximum(high);

					double prev_min = pchart_->GetAxis().GetLeft().GetMinimum();
					if (low <= prev_min)
						pchart_->GetAxis().GetLeft().SetMinimum(low);
				}

				bInitY1_ = false;
			}
		}
		break;

		case EN_AXIS_VERTICAL_RIGHT:
		{
			if (IsAxisLog(CChartWrapper::EN_AXIS_VERTICAL_RIGHT) && bAutoscale_)
			{
				double high = GetNearestLogPoint(ymax);
				double low = ymin;
				if (bInitY2_)
				{
					pchart_->GetAxis().GetRight().SetMaximum(high);
					pchart_->GetAxis().GetRight().SetMinimum(low);
				}
				else
				{
					double prev_max = pchart_->GetAxis().GetRight().GetMaximum();
					if (high >= prev_max)
						pchart_->GetAxis().GetRight().SetMaximum(high);

					double prev_min = pchart_->GetAxis().GetRight().GetMinimum();
					if (low <= prev_min)
						pchart_->GetAxis().GetRight().SetMinimum(low);
				}

				bInitY2_ = false;
			}
		}
		break;
	}
}

bool CChartWrapper::SetVerticalAxis(const enAxis axis,const int plot_id)
{
	if (plot_id < 0)
		return false;

	
	switch(axis)
	{
		case EN_AXIS_VERTICAL_LEFT:
			pchart_->Series(plot_id).SetVerticalAxis(EN_VERTICAL_AXIS_LEFT);
			pchart_->Series(plot_id-1).SetVerticalAxis(EN_VERTICAL_AXIS_LEFT);
			break;

		case EN_AXIS_VERTICAL_RIGHT:
			pchart_->Series(plot_id).SetVerticalAxis(EN_VERTICAL_AXIS_RIGHT);	
			pchart_->Series(plot_id-1).SetVerticalAxis(EN_VERTICAL_AXIS_RIGHT);
			set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
			break;
			
	}

	return true;
}

CChartWrapper::enAxis CChartWrapper::GetVerticalAxis(const int plot_id)
{
	if (plot_id < 0)
		return EN_AXIS_NONE;
	
	long yAxis = pchart_->Series(plot_id).GetVerticalAxis();
	switch(yAxis)
	{
		case 0:
			return EN_AXIS_VERTICAL_LEFT;

		case 1:
			return EN_AXIS_VERTICAL_RIGHT;

		default:
			return EN_AXIS_VERTICAL_LEFT;
			break;
	}	
}

bool CChartWrapper::SetRange(const enAxis axis,
							  double lower_limit,
							  double upper_limit)
{
	bool status = true;
	
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
		{
			if (pchart_->GetAxis().GetBottom().GetMinimum() > upper_limit)
			{
				pchart_->GetAxis().GetBottom().SetMinimum(lower_limit);
				pchart_->GetAxis().GetBottom().SetMaximum(upper_limit);
			}
			else
			{
				pchart_->GetAxis().GetBottom().SetMaximum(upper_limit);
				pchart_->GetAxis().GetBottom().SetMinimum(lower_limit);
			}

			pchart_->GetAxis().GetBottom().GetLabels().SetStyle(talValue);
		}			   
		break;

		case EN_AXIS_HORIZONTAL_TOP:
		{
			if (pchart_->GetAxis().GetTop().GetMinimum() > upper_limit)
			{
				pchart_->GetAxis().GetTop().SetMinimum(lower_limit);
				pchart_->GetAxis().GetTop().SetMaximum(upper_limit);
			}
			else
			{
				pchart_->GetAxis().GetTop().SetMaximum(upper_limit);
				pchart_->GetAxis().GetTop().SetMinimum(lower_limit);
			}
			pchart_->GetAxis().GetTop().GetLabels().SetStyle(talValue);
			set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
		}
		break;

		case EN_AXIS_VERTICAL_LEFT:
		{
			if(pchart_->GetAxis().GetLeft().GetMinimum() > upper_limit)
			{
				pchart_->GetAxis().GetLeft().SetMinimum(lower_limit);
				pchart_->GetAxis().GetLeft().SetMaximum(upper_limit); 
			}
			else
			{
				pchart_->GetAxis().GetLeft().SetMaximum(upper_limit); 
				pchart_->GetAxis().GetLeft().SetMinimum(lower_limit);
			}
			   
			pchart_->GetAxis().GetLeft().GetLabels().SetStyle(talValue);
		}
		break;

		case EN_AXIS_VERTICAL_RIGHT:
			 pchart_->GetAxis().GetRight().SetMinimum(lower_limit);
			 pchart_->GetAxis().GetRight().SetMaximum(upper_limit);
			 pchart_->GetAxis().GetRight().GetLabels().SetStyle(talValue);
			 set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
		break;

		default:
			status = false;			
	}

	return status;
}

void CChartWrapper::RemoveAxis(const enAxis axis)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
		case EN_AXIS_VERTICAL_LEFT:
		break;

		case EN_AXIS_VERTICAL_RIGHT:
		{
			std::set<int>::iterator set_iter = set_axes_.find(EN_AXIS_VERTICAL_RIGHT);
			if(set_iter != set_axes_.end())
				set_axes_.erase(set_iter);
		}
		break;
		case EN_AXIS_HORIZONTAL_TOP:
		{
			std::set<int>::iterator set_iter = set_axes_.find(EN_AXIS_HORIZONTAL_TOP);
			if(set_iter != set_axes_.end())
				set_axes_.erase(set_iter);
		}
		break;
	}
}

void CChartWrapper::DestroyChartWindow()
{
	if (isDynamicallyCreated_ && pchart_->GetSafeHwnd() != NULL) {
		pchart_->SendMessage(WM_CLOSE);
		pchart_->DestroyWindow();
	}
}

int CChartWrapper::GetAxesCount()
{
	return set_axes_.size();
}

int CChartWrapper::GetLinePointPlotCount()
{
	return pchart_->GetSeriesCount() - 1;
}

void CChartWrapper::SetSeriesMarkTip(const int plot_id)
{
	if (plot_id < 0)
		return;

	CToolList tools = pchart_->GetTools();
	tools.Add(tcMarksTip);

	CTools tool = tools.GetItems(0);
	CMarksTipTool mark_tip = tool.GetAsMarksTip();

	VARIANT var;
	var.vt = VT_I1;
	var.bVal = plot_id;

	mark_tip.SetSeries(var);
	mark_tip.SetMouseAction(mtmMove);
	mark_tip.SetDelay(500);
	mark_tip.SetHideDelay(2500);
	
}

void CChartWrapper::SetAxisExpoLabels(const enAxis axis, bool bExpo, bool isLessOne)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
		{	
			if(IsAxisLog(axis))
			{
				if(bExpo)
				{
					pchart_->GetAxis().GetBottom().GetLabels().SetStyle(talValue);
					if(isLessOne)
						pchart_->GetAxis().GetBottom().GetLabels().SetValueFormat("#.####e-000#");
					else
						pchart_->GetAxis().GetBottom().GetLabels().SetValueFormat("#.####e+000#");
				}
				else
				{
					pchart_->GetAxis().GetBottom().GetLabels().SetStyle(talAuto);
				}
			}
			else
			{	
				pchart_->GetAxis().GetBottom().GetLabels().SetStyle(talValue);
				if(bExpo)
				{
					if(isLessOne)
						pchart_->GetAxis().GetBottom().GetLabels().SetValueFormat("#.####e-000#");
					else
						pchart_->GetAxis().GetBottom().GetLabels().SetValueFormat("#.####e+000#");
				}
				else
					pchart_->GetAxis().GetBottom().GetLabels().SetValueFormat("#,##0.#####");
			}
		}
		break;

		case EN_AXIS_VERTICAL_LEFT:
		{
			if(IsAxisLog(axis))
			{
				pchart_->GetAxis().GetLeft().GetLabels().SetStyle(talValue);
				if(bExpo)
				{					
					if(isLessOne)
						pchart_->GetAxis().GetLeft().GetLabels().SetValueFormat("#.##e-00#");
					else
						pchart_->GetAxis().GetLeft().GetLabels().SetValueFormat("#.##e+00#");
				}
				else
				{
					//pchart_->GetAxis().GetLeft().GetLabels().SetStyle(talAuto);
					CString format = pchart_->GetAxis().GetLeft().GetLabels().GetValueFormat();
					if (format != "#.##e-00#")
						pchart_->GetAxis().GetLeft().GetLabels().SetValueFormat("#,##0.#####");
				}
			}
			else
			{
				pchart_->GetAxis().GetLeft().GetLabels().SetStyle(talValue);
				if(bExpo)
				{
					if(isLessOne)
						pchart_->GetAxis().GetLeft().GetLabels().SetValueFormat("#.###e-00#");
					else
						pchart_->GetAxis().GetLeft().GetLabels().SetValueFormat("#.##e+00#");
				}
				else
					pchart_->GetAxis().GetLeft().GetLabels().SetValueFormat("#,##0.####");
			}
		}
		break;

		case EN_AXIS_VERTICAL_RIGHT:
		{
			if(IsAxisLog(axis))
			{
				if(bExpo)
				{
					pchart_->GetAxis().GetRight().GetLabels().SetStyle(talValue);
					if (isLessOne)
						pchart_->GetAxis().GetRight().GetLabels().SetValueFormat("#.##e-00#");
					else
						pchart_->GetAxis().GetRight().GetLabels().SetValueFormat("#.##e+00#");
				}
				else
				{
					pchart_->GetAxis().GetRight().GetLabels().SetStyle(talAuto);					
				}
			}
			else
			{
				pchart_->GetAxis().GetRight().GetLabels().SetStyle(talValue);
				if(bExpo)
				{
					if(isLessOne)
						pchart_->GetAxis().GetRight().GetLabels().SetValueFormat("#.##e-00#");
					else
						pchart_->GetAxis().GetRight().GetLabels().SetValueFormat("#.##e+00#");
				}
				else
					pchart_->GetAxis().GetRight().GetLabels().SetValueFormat("#,##0.#####");

			}			
		}
		break;
	}

}
void CChartWrapper::SetAxisVisible(const enAxis axis, const bool bVisible)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			{
				pchart_->GetAxis().GetBottom().SetVisible(bVisible);
			}
			break;

		case EN_AXIS_HORIZONTAL_TOP:
			{
				pchart_->GetAxis().GetTop().SetVisible(bVisible);
				if(bVisible)
					set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
				else
				{
					std::set<int>::iterator set_iter = set_axes_.find(EN_AXIS_HORIZONTAL_TOP);
					if(set_iter != set_axes_.end())
						set_axes_.erase(set_iter);
				}
			}
		break;

		case EN_AXIS_VERTICAL_LEFT:
			{
				pchart_->GetAxis().GetLeft().SetVisible(bVisible);
			}
		    break;

		case EN_AXIS_VERTICAL_RIGHT:
			{
				pchart_->GetAxis().GetRight().SetVisible(bVisible);
				if(bVisible)
					set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
				else
				{
					std::set<int>::iterator set_iter = set_axes_.find(EN_AXIS_VERTICAL_RIGHT);
					if(set_iter != set_axes_.end())
						set_axes_.erase(set_iter);
				}
			}
			break;              
	}
}

void CChartWrapper::SetAxisAutoscale(const enAxis axis, const bool bAutoscale)
{
	bAutoscale_ = bAutoscale;

	bool flag = bAutoscale;
	if(IsAxisLog(axis) && bAutoscale)
		flag = false;

	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			pchart_->GetAxis().GetBottom().SetAutomatic(flag);
		
			break;

		case EN_AXIS_HORIZONTAL_TOP:
			pchart_->GetAxis().GetTop().SetAutomatic(flag);
			set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
		break;

		case EN_AXIS_VERTICAL_LEFT:
			pchart_->GetAxis().GetLeft().SetAutomatic(flag);
		    break;

		case EN_AXIS_VERTICAL_RIGHT:
			pchart_->GetAxis().GetRight().SetAutomatic(flag);
			set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
			break;              
	}
}

void CChartWrapper::SetFrameStyle(const enFrameStyle style)
{
	switch(style)
	{
		case EN_FRAME_CLASSIC:
		{
			pchart_->GetPanel().GetGradient().SetVisible(TRUE);
			pchart_->GetPanel().GetGradient().SetStartColor(RGB(225,225,225));
			pchart_->GetPanel().GetGradient().SetEndColor(RGB(225,225,225));
			pchart_->GetPanel().GetGradient().SetMidColor(RGB(225,225,225));
			pchart_->GetWalls().GetBack().SetVisible(TRUE);
			pchart_->GetWalls().GetBack().SetColor(RGB(255,255,255)); 
			pchart_->GetWalls().GetBack().SetTransparent(TRUE);
		}
		break;
	}
}

void CChartWrapper::SetAxisFont(const enAxis axis, const CString& fontName, const int fontSize)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			pchart_->GetAxis().GetBottom().GetLabels().GetFont().SetName(fontName);
			pchart_->GetAxis().GetBottom().GetLabels().GetFont().SetSize(fontSize);
			break;

		case EN_AXIS_HORIZONTAL_TOP:
			pchart_->GetAxis().GetTop().GetLabels().GetFont().SetName(fontName);
			pchart_->GetAxis().GetTop().GetLabels().GetFont().SetSize(fontSize);
			set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
		break;

		case EN_AXIS_VERTICAL_LEFT:
			pchart_->GetAxis().GetLeft().GetLabels().GetFont().SetName(fontName);
			pchart_->GetAxis().GetLeft().GetLabels().GetFont().SetSize(fontSize);
		    break;

		case EN_AXIS_VERTICAL_RIGHT:
			pchart_->GetAxis().GetRight().GetLabels().GetFont().SetName(fontName);
			pchart_->GetAxis().GetRight().GetLabels().GetFont().SetSize(fontSize);
			set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
			break;              
	}	
}

void CChartWrapper::SetAxisLabelColor(const enAxis axis ,COLORREF color)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			pchart_->GetAxis().GetBottom().GetAxisPen().SetColor(color);
		break;
		case EN_AXIS_HORIZONTAL_TOP:
			pchart_->GetAxis().GetTop().GetAxisPen().SetColor(color);
		break;
		case EN_AXIS_VERTICAL_LEFT:
			pchart_->GetAxis().GetLeft().GetAxisPen().SetColor(color);
		break;
		case EN_AXIS_VERTICAL_RIGHT:
			pchart_->GetAxis().GetRight().GetAxisPen().SetColor(color);
		break;        
	}
}

void CChartWrapper::SetAxisMajorTickColor(const enAxis axis ,COLORREF color)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
			pchart_->GetAxis().GetBottom().GetTicks().SetColor(color);
		break;

		case EN_AXIS_HORIZONTAL_TOP:
			pchart_->GetAxis().GetTop().GetTicks().SetColor(color);
		break;

		case EN_AXIS_VERTICAL_LEFT:
			pchart_->GetAxis().GetLeft().GetTicks().SetColor(color);
		break;

		case EN_AXIS_VERTICAL_RIGHT:
			pchart_->GetAxis().GetRight().GetTicks().SetColor(color);
		break;        
	}
}

void CChartWrapper::SetFont(CFont *font)
{
	font_.reset(font);
	pchart_->SetFont(font_.get());
}

void CChartWrapper::SetFrameColor(COLORREF color)
{
	pchart_->GetPanel().GetGradient().SetStartColor(color);
	pchart_->GetPanel().GetGradient().SetEndColor(color);
	pchart_->GetPanel().GetGradient().SetMidColor(color);
}

void CChartWrapper::SetCaption(const CString &caption, COLORREF color)
{
	BSTR str = caption.AllocSysString();
	VARIANT var;
	var.vt = VT_BSTR;
	var.bstrVal = str;
	
	pchart_->GetHeader().SetVisible(TRUE);
	pchart_->GetHeader().SetAlignment(2);
	pchart_->GetHeader().GetFont().SetName("Arial");
	pchart_->GetHeader().GetFont().SetSize(9);
	pchart_->GetHeader().GetFont().SetColor(color);
	pchart_->GetHeader().GetText().Clear();
	pchart_->GetHeader().GetText().Add(var);	

	SysFreeString(str);
}

void CChartWrapper::SetPlotLineWidth(const int plot_id,const int width)
{
	if (plot_id < 0)
		return;

	pchart_->Series(plot_id).GetAsLine().GetLinePen().SetWidth(width);
	
	SetPlotPointWidth(plot_id + 1, width);
}

void CChartWrapper::SetPlotLineColor(const int plot_id,COLORREF color)
{
	if (plot_id < 0)
		return;

	if(GetPlotLineStyle(plot_id) == psClear )
		color = RGB(255,255,255);

	pchart_->Series(plot_id).GetAsLine().GetLinePen().SetVisible(TRUE);
	pchart_->Series(plot_id).GetAsLine().GetLinePen().SetColor(color);
	pchart_->Series(plot_id).SetColor(color);

	std::map<int,COLORREF>::iterator map_iter = map_plot_line_color_.find(plot_id);
	if(map_iter != map_plot_line_color_.end())
		map_iter->second = color;
	else
		map_plot_line_color_.insert(std::make_pair(plot_id, color));
}	

void CChartWrapper::SetPlotLineStyle(const int plot_id,const int style)
{
	if (plot_id < 0)
		return;

	lineStyle_ = style;

	pchart_->Series(plot_id).GetAsLine().GetLinePen().SetStyle(psSolid);
}

long CChartWrapper::GetPlotLineStyle(const int plot_id)
{
	if (plot_id < 0)
		return -1L;

	return pchart_->Series(plot_id).GetAsLine().GetLinePen().GetStyle();
}

void CChartWrapper::SetPlotPointWidth(const int plot_id, int width)
{
	if (plot_id < 0)
		return;

	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetVisible(TRUE);
	if(width <= 2)
	{
		FHozFactor_ = 2;
		FVerFactor_ = 2;
	}
	else
	{
		FHozFactor_ = 3 ;
		FVerFactor_ =  3 ;
	}
	if(width < 4)
	{
		pchart_->Series(plot_id).GetAsPoint().GetPointer().SetHorizontalSize(FHozFactor_);
		pchart_->Series(plot_id).GetAsPoint().GetPointer().SetVerticalSize(FVerFactor_);
		
		pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetWidth(width);
	}
	else
		pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetWidth(width);//2//bug#6
}

void CChartWrapper::SetPlotPointStyle(const int plot_id,const long style)
{
	if (plot_id < 0)
		return;

	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetVisible(TRUE);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetVisible(TRUE);
	
	nsSTAr::nsChartDataType::enChartPointerStyle pointerStyle = nsSTAr::nsChartDataType::PointerNone;
	long penStyle = psClear;
	long brushStyle = bsSolid;

	nPointStyle_ = style;

	pointerStyle = nsSTAr::nsChartDataType::PointerX;
	{
		pchart_->Series(plot_id).GetAsPoint().GetPointer().SetHorizontalSize(FHozFactor_);
		pchart_->Series(plot_id).GetAsPoint().GetPointer().SetVerticalSize(FVerFactor_);
		pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetWidth(2);
		penStyle = psSolid;
	}

	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetStyle(pointerStyle);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().GetBrush().SetStyle(brushStyle);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetStyle(penStyle);
}

void CChartWrapper::SetPlotPointColor(const int plot_id,COLORREF color)
{
	if (plot_id < 0)
		return;

	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetVisible(TRUE);	
	pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetVisible(TRUE);	

	switch(nPointStyle_)
	{
		case 0:
			color = RGB(255,255,255);
			pchart_->Series(plot_id).SetColor(RGB(255,255,255));
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetBrush().SetColor(RGB(255,255,255));
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetColor(RGB(255,255,255));
		break;

		case 1:	
		case 4:
		case 9:
		case 11:
		case 19:
		case 22:
			pchart_->Series(plot_id).SetColor(color);
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetBrush().SetColor(RGB(255,255,255));
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetColor(color);
			break;

		case 7:
		case 8:
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetBrush().SetColor(color);
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetColor(color);
			pchart_->Series(plot_id).SetColor(RGB(255,0,0));
			color = RGB(255,255,255);
			break;

		default:
			pchart_->Series(plot_id).SetColor(color);
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetColor(color);
			pchart_->Series(plot_id).GetAsPoint().GetPointer().GetBrush().SetColor(color);
			break;
	}

	std::map<int, COLORREF>::iterator map_iter = map_plot_point_color_.find(plot_id);
	if(map_iter != map_plot_point_color_.end())
		map_iter->second = color;
	else
		map_plot_point_color_.insert(std::make_pair(plot_id, color));
}

void CChartWrapper::ClearData()
{
	pchart_->ClearChart();
}

void CChartWrapper::SetAxisTicks(const enAxis axis)
{
	switch(axis)
	{
		case EN_AXIS_HORIZONTAL_BOTTOM:
		{
			pchart_->GetAxis().GetBottom().GetGridPen().SetVisible(TRUE);
			pchart_->GetAxis().GetBottom().GetGridPen().SetStyle(0);
			pchart_->GetAxis().GetBottom().GetTicks().SetVisible(TRUE);
			pchart_->GetAxis().GetBottom().GetTicks().SetWidth(1);
			pchart_->GetAxis().GetBottom().GetAxisPen().SetVisible(FALSE);
			pchart_->GetAxis().GetBottom().GetMinorGrid().SetVisible(TRUE);
			pchart_->GetAxis().GetBottom().SetMinorTickCount(1);
			pchart_->GetAxis().GetBottom().GetTicksInner().SetVisible(FALSE);
			pchart_->GetAxis().GetBottom().GetLabels().GetFont().SetName("Arial");
			pchart_->GetAxis().GetBottom().GetLabels().GetFont().SetSize(9);
		}
		break;

		case EN_AXIS_HORIZONTAL_TOP:
		{
			pchart_->GetAxis().GetTop().GetGridPen().SetVisible(TRUE);
			pchart_->GetAxis().GetTop().GetTicks().SetVisible(TRUE);
			pchart_->GetAxis().GetTop().GetTicks().SetWidth(1);
			pchart_->GetAxis().GetTop().GetAxisPen().SetVisible(FALSE);
			pchart_->GetAxis().GetTop().GetMinorGrid().SetVisible(TRUE);
			pchart_->GetAxis().GetTop().GetTicksInner().SetVisible(FALSE);
			pchart_->GetAxis().GetTop().GetLabels().GetFont().SetName("Arial");
			pchart_->GetAxis().GetTop().GetLabels().GetFont().SetSize(9);
			set_axes_.insert(EN_AXIS_HORIZONTAL_TOP);
		}
		break;

		case EN_AXIS_VERTICAL_LEFT:
		{
			pchart_->GetAxis().GetLeft().GetGridPen().SetVisible(TRUE);
			pchart_->GetAxis().GetLeft().GetGridPen().SetStyle(0);
			pchart_->GetAxis().GetLeft().GetTicks().SetVisible(TRUE);
			pchart_->GetAxis().GetLeft().GetTicks().SetWidth(1);
			pchart_->GetAxis().GetLeft().GetAxisPen().SetVisible(FALSE);
			pchart_->GetAxis().GetLeft().GetMinorGrid().SetVisible(TRUE);
			pchart_->GetAxis().GetLeft().SetMinorTickCount(1);
			pchart_->GetAxis().GetLeft().GetMinorTicks().SetVisible(FALSE);
			pchart_->GetAxis().GetLeft().GetTicksInner().SetVisible(FALSE);
			pchart_->GetAxis().GetLeft().GetLabels().GetFont().SetName("Arial");
			pchart_->GetAxis().GetLeft().GetLabels().GetFont().SetSize(9);
		}
		break;

		case EN_AXIS_VERTICAL_RIGHT:
		{
			pchart_->GetAxis().GetRight().GetGridPen().SetVisible(false);
			pchart_->GetAxis().GetRight().GetTicks().SetVisible(TRUE);
			pchart_->GetAxis().GetRight().GetTicks().SetWidth(1);
			pchart_->GetAxis().GetRight().GetAxisPen().SetVisible(FALSE);
			pchart_->GetAxis().GetRight().GetMinorGrid().SetVisible(false);
			pchart_->GetAxis().GetRight().SetMinorTickCount(1);
			pchart_->GetAxis().GetRight().GetMinorTicks().SetVisible(FALSE);
			pchart_->GetAxis().GetRight().GetTicksInner().SetVisible(FALSE);
			pchart_->GetAxis().GetRight().GetLabels().GetFont().SetName("Arial");
			pchart_->GetAxis().GetRight().GetLabels().GetFont().SetSize(9);
			set_axes_.insert(EN_AXIS_VERTICAL_RIGHT);
		}
		break;        
	}	
}

void CChartWrapper::SetPlotAreaColor(const COLORREF color)
{
	pchart_->GetWalls().GetBack().SetColor(color);
	pchart_->GetWalls().GetBack().GetGradient().SetEndColor(color);
	pchart_->GetWalls().GetBack().GetGradient().SetStartColor(color);
	pchart_->GetWalls().GetBack().GetGradient().SetUseMiddleColor(TRUE);
	pchart_->GetWalls().GetBack().SetTransparent(FALSE);
}

HWND CChartWrapper::GetSafeHwnd()
{
	return pchart_->GetSafeHwnd();
}

void CChartWrapper::SetWindowPos(CWnd* pwnd, int x, int y, int cx, int cy, UINT flags)
{
	//pchart_->SetWindowPos(pwnd, x, y, cx, cy, flags);
	pchart_->SetLeft(x);
	pchart_->SetTop(y);
	pchart_->SetWidth(cx);
	pchart_->SetHeight(cy);
}

void CChartWrapper::GetMinMaxValue1D(Array1D_Double Vec_DataX,
	Array1D_Double Vec_DataY,
	double &xmin,
	double &xmax,
	double &ymin,
	double &ymax)
{
	xmin = (std::numeric_limits<double>::infinity)();
	xmax = -(std::numeric_limits<double>::infinity)();
	ymin = (std::numeric_limits<double>::infinity)();
	ymax = -(std::numeric_limits<double>::infinity)();

	for (int i = 0; i < (int)Vec_DataX.size(); i++)
	{
		if (xmax < Vec_DataX[i])
			xmax = Vec_DataX[i];

		if (xmin > Vec_DataX[i])
			xmin = Vec_DataX[i];
	}

	for (int i = 0; i < (int)Vec_DataY.size(); i++)
	{
		if (ymax < Vec_DataY[i])
			ymax = Vec_DataY[i];

		if (ymin > Vec_DataY[i])
			ymin = Vec_DataY[i];
	}
}


void CChartWrapper::SetPlotVisible(const int plot_id, const bool status)
{
	if (plot_id < 0)
		return;

	pchart_->Series(plot_id-1).GetAsLine().GetLinePen().SetVisible(status);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().GetPen().SetVisible(status);
	pchart_->Series(plot_id).GetAsPoint().GetPointer().SetVisible(status);
	pchart_->Series(plot_id-2).SetActive(status);
	pchart_->Series(plot_id-1).SetActive(status);
	pchart_->Series(plot_id).SetActive(status);
}