#pragma once
#include <set>
#include <map>
#include <vector>
#include <memory>
#include "helpers\TeeChartDefines.h"
#include "datatypes.h"

class CTChart;

class CSmithSeries;
class CPolarSeries;

namespace nsSTAr
{
	namespace nsChartDataType
	{
		typedef std::vector<double> CReal64Vector;

		struct CColor
		{
			COLORREF color_;

			CColor(const long r,const long g, const long b) : color_(RGB(r,g,b)) {}

			CColor(const COLORREF color) : color_(color) {}

			CColor& operator=(const CColor &source)
			{
				color_ = source.color_;
			}

			COLORREF getColor() 
			{
				return color_;
			}
		};

		enum enChartLineStyle
		{
			LineNone                  = psClear,
            // Solid line.
            LineSolid                 = psSolid,
            // Dash line.
            LineDash                  = psDash,
            // Dotted line.
            LineDot                   = psDot,
            // Dash-dot line.
            LineDashDot               = psDashDot,
            // Dash-dot-dot line.
            LineDashDotDot            = psDashDotDot,
            // Step centered line.
            LineSmallDot			  =	psSmallDots,

			LineStepXY				  = 8,
			
			LineStepYX				  = 9
			
		};

		enum enChartPointStyle
		{
			PointNone                 = 0,
            // Empty square.
            PointEmptySquare          = 1,
            // Solid square.
            PointSolidSquare          = 2,
            // Asterisk
            PointAsterisk             = 3,
            // Dotted empty square.
            PointDottedEmptySquare    = 4,
            // Dotted solid square.
            PointDottedSolidSquare    = 5,
            // Solid diamond.
            PointSolidDiamond         = 6,
            // Empty square with X.
            PointEmptySquareWithX     = 7,
            // Empty square with cross.
            PointEmptySquareWithCross = 8,
            // Empty circle.
            PointEmptyCircle          = 9,
            // Solid circle.
            PointSolidCircle          = 10,
            // Dotted empty circle.
            PointDottedEmptyCircle    = 11,
            // Dotted solid circle.
            PointDottedSolidCircle    = 12,
            // X
            PointX                    = 13,
            // Bold X.
            PointBoldX                = 14,
            // Small X.
            PointSmallX               = 15,
            // Cross
            PointCross                = 16,
            // Bold cross.
            PointBoldCross            = 17,
            // Small cross.
            PointSmallCross           = 18,
            // Small empty square.
            PointSmallEmptySquare     = 19,
            // Small solid square.
            PointSmallSolidSquare     = 20,
            // Simple dot.
            PointSimpleDot            = 21,
            // Empty diamond.
            PointEmptyDiamond         = 22 			
		};	

		enum enChartPointerStyle
		{
			PointerNone             = -1,
			PointerSquare			= psRectangle,
			PointerCircle				= psCircle,
			PointerCross				= psCross,
            PointerX					= psDiagCross,
            PointerAsterisk         = psStar,
            PointerDiamond			= psDiamond,   
            PointerSimpleDot          = psSmallDot
		};
	}
}

class CChartWrapper
{
public:
	
	CChartWrapper(CTChart *chart = 0);
	~CChartWrapper(void);

	static const int LINE_TYPE;
	static const int BAR_TYPE;
	static const int H_BAR_TYPE;
	static const int POINT_TYPE;
	static const int BEZIER_TYPE;
	static const int AREA_TYPE;
	static const int SMITH_TYPE;
	static const int POLAR_TYPE;

	std::auto_ptr<CFont> font_;
	std::map<int,COLORREF> map_plot_line_color_;
	std::map<int,COLORREF> map_plot_point_color_;
	std::map<int,COLORREF> map_plot_area_color_;

	long nPointStyle_;

	enum enAxis
	{
		EN_AXIS_NONE		 = -1,
		EN_AXIS_VERTICAL_LEFT = 0,
		EN_AXIS_HORIZONTAL_TOP = 1,
		EN_AXIS_VERTICAL_RIGHT = 2,
		EN_AXIS_HORIZONTAL_BOTTOM = 3		
	};

	enum enVerticalAxis
	{
		EN_VERTICAL_AXIS_LEFT = 0,
		EN_VERTICAL_AXIS_RIGHT = 1,
		EN_VERTICAL_AXIS_BOTH = 2		
	};

	enum enImageFormat
	{
		EN_FORMAT_NONE = 0,
		EN_FORMAT_BMP,
		EN_FORMAT_JPG,
		EN_FORMAT_PNG
	};

	enum enFrameStyle
	{
		EN_FRAME_NONE = 0,
		EN_FRAME_CLASSIC
	};

	enum enLableStyle
	{
		EN_LABLE_NONE = 0,
		EN_LABLE_NORMAL,
		EN_LABLE_NEGETIVE_EXPONENTIAL,
		EN_LABLE_POSITIVE_EXPONENTIAL
	};

	std::set<int>	set_axes_;
	enLableStyle xExpo_;
	enLableStyle y1Expo_;
	enLableStyle y2Expo_;

	std::map<int, std::shared_ptr<CSmithSeries> > mapSmithChartWnd_;
	std::map<int, std::shared_ptr<CPolarSeries> > mapPolarChartWnd_;
	

	UINT res_id_;

	CTChart* GetTChart();
	bool CreateTChart(const CRect &rect,
					CWnd *parent,
					const UINT res_id);
	
	bool SetTChartProperty();
	void GetTChartWindowRect(CRect& rect);
	void TChartInvalidate();

	void SetPlotVisible(const int plot_id, const bool status);

	bool IsAxisLog(const enAxis axis);	//v
	void SetAxisLog(const enAxis axis, const bool isLog);
	//void ResetAxis();

	int AddPlot(const CString &strParameter,const double width,const long axis = 0);
	int AddPointPlot(const CString &strParameter,
						   const double width,
						   const long axis = 0);
	int AddAreaPlot(const CString &strParameter,
						   const double width, 
						   const long axis);
	////int AddLinePointPlot(const CString &strParameter,
	////					        const double width,
	////							const long axis = 0);
	int AddLinePointPlotEx(const CString &strParameter,		//v
						        const double width,
								const long axis = 0);
	////int AddBarPlot(const CString &strParameter);
	////int AddHBarPlot(const CString &strParameter);
	////int AddSmithPlot(const CString &strParameter);
	////CSmithSeries* GetSmithChart(const int series_id);
	////CPolarSeries* GetPolarChart(const int series_id);
	////int AddPolarPlot(const CString &strParameter);
	void SetAxisTitle(const enAxis axis,const CString &title);
	//void RemovePlot(const int plot_id);
	//void ClearSeries(const int plot_id);
	void RemoveAllPlots();	//v
	//void SetAxisTickLeft(const enAxis axis,const bool status);
	//void SetAxisTickRight(const enAxis axis,const bool status);

	////void GetMinMaxValue(nsSTAr::nsChartDataType::CReal64Vector Vec_DataX,
	////					nsSTAr::nsChartDataType::CReal64Vector Vec_DataY,
	////					double &xmin,
	////					double &xmax,
	////					double &ymin,
	////					double &ymax);

	void GetMinMaxValue1D(Array1D_Double Vec_DataX,
		Array1D_Double Vec_DataY,
		double &xmin,
		double &xmax,
		double &ymin,
		double &ymax);


	////void CheckExponentialFormat(const int plot_id,
	////							const double xmin,
	////							const double xmax,
	////							const double ymin,
	////							const double ymax);

	////void PlotLinePointXvsY(const int plot_id,
	////			const std::vector<double> &vec_xval,
	////			const std::vector<double> &vec_yval);

	////void PlotLineXvsY(const int plot_id,
	////				 const std::vector<double> &vec_xval,
	////				 const std::vector<double> &vec_yval);

	void PlotLineXvsY1D(const int plot_id,
		const Array1D_Double &vec_xval,
		const Array1D_Double &vec_yval);


	////void PlotLineXvsYArray(const int plot_id,
	////	 std::vector<double> &vec_xval,
	////	 std::vector<double> &vec_yval);

	////void PlotPointXvsY(const int plot_id,
	////					 const std::vector<double> &vec_xval,
	////					 const std::vector<double> &vec_yval);

	void PlotPointXvsY1D(const int plot_id,
		const Array1D_Double &vec_xval,
		const Array1D_Double &vec_yval);

	////void PlotAreaXvsY(const int plot_id,
	////				 const std::vector<double> &vec_xval,
	////				 const std::vector<double> &vec_yval);

	void PlotXY(const int plot_id,
				const double x,
				const double y,
				const COLORREF color);
	////void PlotXY2(const int plot_id,
	////			const double x,
	////			const double y,
	////			const COLORREF color);

	////void PlotHBarXY(const int plot_id,
	////			const double x,
	////			const double y,
	////			const COLORREF color);

	////void PlotXY(const int plot_id,
	////			const int x,
	////			const double y,
	////			const COLORREF color);
	////void PlotXY(const int plot_id, 
	////					   const int x,
	////					   const double y) ;

	////void PlotSmith(const int plot_id,
	////				const long index,
	////				const double real_part,
	////				const double imag_part);

	void SetScaleX(const int plot_id, const double x);
	////void SetScaleY(const int plot_id, const double y);	
	void SetScaleY(const int plot_id, const double ymin, const double ymax);
	
	////bool GetRange(const enAxis axis,
	////			  double &lower_limit,
	////			  double &upper_limit);
	
	 ////BOOL     IsAllLegendsSelected();
	 ////void DisplayAllPlots(BOOL bShow);
	 //void SetAxisMode(const enAxis axis,BOOL bLogChecked, BOOL bAutoChecked);
	 bool SetRange(const enAxis axis,
				  double lower_limit,
				  double upper_limit);
	 bool SetVerticalAxis(const enAxis axis,const int plot_id);	//v
	 enAxis GetVerticalAxis(const int plot_id);
	 ////bool SetInterval(const enAxis axis,
		////			  double interval);
	 ////void SetMinorTickCount(const enAxis axis,const int value);
	 ////void SetBarWidth(const int plot_id,
		////		  const double width);
	 ////void SaveChart(LPCTSTR FileName);
	 ////CString GetTitle(const int PLOT_ID) ;
	 ////BOOL IsLegendSelected(CString strLegendString);
	 ////double getPointYValue(const int SeriesIndx,const int ValueIndx);
	 ////double GetXPointValue(const int SeriesIndx, long Index);
	 ////double GetYPointValue(const int SeriesIndx, long Index);
	 ////long GetPointXAxisVal(long Xpoint);
	 ////long GetPointYAxisVal(double Ypoint);
	 ////void SaveChart(const enImageFormat format,
		////			CString &file_name);
	 ////void SetBarBorderColor(const int plot_id,COLORREF colr);
	 ////void SetMinMaxAxis(const enAxis axis,
		////			  BOOL bAutoChecked, 
		////			  BOOL bMin = true, 
		////			  BOOL bMax = true);
	 void DestroyChartWindow();
	 int GetAxesCount();
	 void RemoveAxis(const enAxis axis);
	 ////int GetPlotCount();
	 ////int GetPointPlotCount();
	 int GetLinePointPlotCount();	//v
	 void SetAxisVisible(const enAxis axis, const bool bVisible);
	 void SetAxisAutoscale(const enAxis axis, const bool bAutoscale);
	 ////bool IsAxisAutoscale(const enAxis axis);
	 void SetFrameStyle(const enFrameStyle style);
	 void SetAxisFont(const enAxis axis, const CString& fontName, const int fontSize);
	 void SetAxisTicks(const enAxis axis);
	 void SetPlotAreaColor(const COLORREF color);
	 void SetPlotLineWidth(const int plot_id,const int width);	//v
	 ////long GetPlotLineWidth(const int plot_id);
	 void SetPlotLineStyle(const int plot_id,const int style);	//v
	 long GetPlotLineStyle(const int plot_id);
	 void SetPlotPointWidth(const int plot_id, int width);
	 ////int GetPlotPointWidth(const int plot_id);
	 void SetPlotPointStyle(const int plot_id,const long style);	//v
	 ////long GetPlotPointStyle(const int plot_id);
	 void SetAxisLabelColor(const enAxis axis ,COLORREF color);
	 void SetAxisMajorTickColor(const enAxis axis ,COLORREF color);
	 ////void SetAxisGridColor(const enAxis axis ,COLORREF color);
	 void SetFont(CFont *font);
	 void SetFrameColor(COLORREF color);
	 void SetCaption(const CString &caption, COLORREF color);
	 void SetPlotLineColor(const int plot_id,COLORREF color);	//v
	 COLORREF GetPlotLineColor(const int plot_id);
	 void SetPlotPointColor(const int plot_id,COLORREF color);	//v
	 ////COLORREF GetPlotPointColor(const int plot_id);
	 ////void SetPlotAreaFillBaseColor(const int plot_id,COLORREF color);
	 ////COLORREF GetPlotAreaFillBaseColor(const int plot_id);
	 ////void SetPlotAreaLineColor(const int plot_id,COLORREF color);
	 ////COLORREF GetPlotAreaLineColor(const int plot_id);
	 ////void SetAreaFillBase(const int plot_id, const bool isFillBase);
	 ////bool GetAreaFillBase(const int plot_id);
	 ////void SetAreaLine(const int plot_id, const bool isLines, const bool isFillBase);
	 ////long GetAreaLine(const int plot_id);
	 void ClearData();

	 HWND GetSafeHwnd();
	 void SetWindowPos(CWnd* pwnd, int x, int y, int cx, int cy, UINT flags);
	
	 void SetAxisExpoLabels(const enAxis axis, bool val = false, bool isLessOne = false);

	 void SetSeriesMarkTip(const int plot_id);	

private:
	CTChart *pchart_;
	bool isDynamicallyCreated_;
	int plot_count_;
	std::set<int>		set_plot_;
	std::set<int>		set_point_plot_;
	std::set<int>		set_line_point_plot_;
	std::map<CString, int>  mapOfLegends_;
	std::map<double,std::pair<long,long> >	mapIndex_;
	int FHozFactor_;
	int FVerFactor_;
	int lineStyle_;
	bool isAreaActive_;
	bool bAutoscale_;
	int prev_plotid_;
	bool bInitX_;
	bool bInitY1_;
	bool bInitY2_;
};

