#pragma once
#include <afxtempl.h>
#include <map>
#include <queue>
#include <memory>
class CStAnalyzerDlg;
class CStGIS_AnalyzerData;
class CStGIS_GraphData;
class CStGIS_TextData;
class CStAnalyzerThread;


class CTesterAnalyzerManager
{
public:
	CTesterAnalyzerManager(void);
	~CTesterAnalyzerManager(void);

	static const std::string TESTER1_;

	struct CTesterAnalyzer
	{
		CTesterAnalyzer(const std::string &testerid);

		typedef std::queue<CStGIS_GraphData> GRAPHDATAQUEUE;
		GRAPHDATAQUEUE que_graphdata;

		typedef std::queue<CStGIS_TextData> TEXTDATAQUEUE;
		TEXTDATAQUEUE que_textdata;

		typedef std::queue<CStGIS_AnalyzerData> ANALYZERDATAQUEUE;
		ANALYZERDATAQUEUE que_analyzerdata;

	};

	std::map<std::string, std::shared_ptr<CTesterAnalyzer> > map_tester_results_;
	typedef std::map<std::string, std::shared_ptr<CTesterAnalyzer> >::iterator IterMap;
	typedef std::map<std::string, std::shared_ptr<CTesterAnalyzer> >::const_iterator IterMapConst;

	CTesterAnalyzer& GetTesterAnalyzer(const std::string& testerid);	
};

////struct CPlotTextInfo
////{
////	enum E_GIS_Command_Set gis_type_;
////	int sequenceID_;
////	std::string paramName1_;
////	int nSingleValue_;
////	int nUpdateMode_;
////	double dSingleValue_;
////	Array1D_Double temp_;
////	std::string paramName2_;
////	Array1D_Double temp1_;
////	std::string paramName3_;
////	Array1D_Double temp2_;
////	Array2D_Double temp3_;
////
////	CPlotTextInfo()	{}
////
////	~CPlotTextInfo()	{}
////};
////
////struct CPlotGraphInfo
////{
////	enum E_GIS_Command_Set gis_cmd_;
////	int nSequenceID_;
////	CStAxisProperty stObjProp_;
////	CStLineProperty stObjLine_;
////	int nGraphDataType_;
////
////	double dXSingleXY1Value_;
////	double dXSingleXY2Value_;
////	double dXSingleXValue_;
////
////	// data storage 
////	Array1D_Double vec_XAxisValues_;
////	Array1D_Double vec_Y1AxisValues_;
////	Array1D_Double vec_Y2AxisValues_;
////
////	Array2D_Double vec_2DY1AxisValues_;
////	Array2D_Double vec_2DY2AxisValues_;
////
////	int nLineColor_;
////	int nLine2Color_;
////	int nDataUpdateMode_; // 0 - Append -, 1 Clear For Graph 
////	int nXYColor_;
////	int nXY2Color_;
////
////	CPlotGraphInfo()	{}
////
////	~CPlotGraphInfo()	{}
////};


class CStAnalyzerManager
{	
public:
	
	int m_nPrefCount;
	// Test 
	
	static CStAnalyzerManager* pStAnalyzerWindowDispatcher;	// single copy of object
	CMap<CString, LPCTSTR, int, int&> arrMapGisCommands;

	std::map<std::pair<std::string, std::string>, CStAnalyzerDlg*> m_aMapGISWndToAlgo;
	//std::map<std::pair<std::string, std::string>, CStAnalyzerThread*> m_aMapGISWndToAlgo;

	typedef std::map<std::pair<std::string, std::string>, std::shared_ptr<CStAnalyzerThread>> ThreadMap;
	ThreadMap mapAnlyzerThreads_;


	// Construction/destructions 
	CStAnalyzerManager(void);
	virtual ~CStAnalyzerManager(void);

private:

	bool IsAnalyzerWindowExist (LPCTSTR tzTest, LPCTSTR tzTester) const;
    std::string previous_test_name_;
	std::string previous_tester_id_;

public :
	
	// Functionalities 
	void CallAnalyzerWndSpecificFunc(CStGIS_AnalyzerData *pData);
	void CallGraphSpecificFunc(CStGIS_GraphData *pData);
	void CallTextSpecificFunc(CStGIS_TextData *pData);


	// Methods due to redesign
	void HandleAnalyzerWndSpecificFunc(std::auto_ptr<CStGIS_AnalyzerData>& pData);
	void HandleGraphSpecificFunc(std::auto_ptr<CStGIS_GraphData>& pData);
	void HandleTextSpecificFunc(std::auto_ptr<CStGIS_TextData>& pData);
	void HandleAnalyzerWindowMenu(const std::string& strTestName, const std::string& strTesterID);
	void CreateAnalyzerThread(const std::string& strTestName, const std::string& strTesterID, int nSizeInPerc, int nTextCount, int nGraphcount, bool bCreateForeGround);
	void CreateAnalyzerThread(const std::string& strTestName, const std::string& strTesterID, CRect rect, int nTextCount, int nGraphcount);
	CStAnalyzerThread* CStAnalyzerManager::GetAnalyzerThread(const std::string& strTestName, const std::string& strTesterID);
	void RemoveAnalyzerThread(const std::string& strTestName, const std::string& strTesterID, bool bForceClose);
	void SetAnalyzerThreadDialogMenu(const std::string& strTestName, const std::string& strTesterID, const int nButtonClose);
	void RemoveAllAnalyzerThreads(void);
	void SetPreferenceWindowCount(void);

	void CreateAnalyzerWindow(LPCTSTR tzTest, LPCTSTR tzTester, CRect rect, int nTextCount, int nGraphcount);
	void CreateAnalyzerWindow(LPCTSTR tzTest, LPCTSTR tzTester, int nSizeInPerc, int nTextCount, int nGraphcount);
	void RemoveAnalyzerWindow(LPCTSTR tzTest, LPCTSTR tzTester, CStAnalyzerDlg* pGraphWnd);
	void RemoveAnalyzerWindow(LPCTSTR tzTest, LPCTSTR tzTester, bool is_force_close = false);
	void CloseAnalyzerWindow(LPCTSTR tzTest, LPCTSTR tzTester);
	void SetAnalyzerWindow(LPCTSTR tzTest, LPCTSTR tzTester,const int nButtonClose);
	void CloseAllAnalyzerWindow();
	void RemoveAnalyzerWindow(bool is_force_close = false);

	CStAnalyzerDlg* GetAnalyzerWnd(LPCTSTR tzTest, LPCTSTR tzTester);
	
	inline static CStAnalyzerManager* CreateAnalyzerWindowDispatcher()
	{
		if (pStAnalyzerWindowDispatcher!=0)
			return pStAnalyzerWindowDispatcher;

		pStAnalyzerWindowDispatcher = new CStAnalyzerManager();
		return pStAnalyzerWindowDispatcher;
	}

	//--------------------------------------------------
	// DeleteAnalyzerWindowDispatcher function
	//--------------------------------------------------
	// * Purpose
	// Delete DeleteAnalyzerWindowDispatcher object. Allows only 1 copy to exist
	//--------------------------------------------------
	inline static void DeleteAnalyzerWindowDispatcher()
	{
		if (pStAnalyzerWindowDispatcher != 0)
			delete pStAnalyzerWindowDispatcher;

		pStAnalyzerWindowDispatcher = 0;
	}

	//--------------------------------------------------
	// IsCreated function
	//--------------------------------------------------
	// * Purpose
	// Check if CStAnalyzerManager object exists
	//
	// * Return value
	// true if object has been created
	// false if object is not created
	//--------------------------------------------------
	inline static  bool IsCreated()
	{
		return (pStAnalyzerWindowDispatcher != 0);
	}

	// Functions

private:

	CCriticalSection csMapAnalyzerThreads_;




};

